obs = obslua

NUMBER_OF_LIGHTS = 8
working_settings = nil
current_cmdline = ""
current_tallystring = ""
tallycli = "TallyLightUSB"

function script_description()
    return [[
Configuration for <a href="http://tally-lights.com">Tally-lights,LLC</a> USB Controller<br>Version 1.0
]]
end

function script_defaults(settings)
	
end


function list_sources()
--	obs.script_log(obs.LOG_INFO, "load_input_settings: ")
    local usable_source_names = {}
	local sources = obs.obs_enum_sources()
	if sources ~= nil then
		for _, source in ipairs(sources) do
			local source_name = obs.obs_source_get_name(source)
            local source_type = obs.obs_source_get_type(source)
            if source_type ~= 3 then -- Ignore groups, which should up as sources
				if source_name ~= "Desktop Audio" and source_name ~= "Mic/Aux" then
					table.insert(usable_source_names, source_name)
--					obs.script_log(obs.LOG_INFO, "s: " .. source_name)
--					obs.script_log(obs.LOG_INFO, "st: " .. source_type)
				end
            end
		end
	end
	obs.source_list_release(sources)
    table.sort(usable_source_names)
    return usable_source_names
end

function script_update(settings)
--	obs.script_log(obs.LOG_INFO, "started ")
    working_settings = settings
end

function script_properties()
    local usable_source_names = list_sources()
	local props = obs.obs_properties_create()
    for _, source_name in ipairs(usable_source_names) do
        obs.obs_properties_add_int(props, source_name, source_name, 0, NUMBER_OF_LIGHTS, 1)
    end
	return props
end

function script_load(settings)
    obs.timer_add(refresh_state, 100)
end

function script_unload()
    handle_exit()
    timer_remove(refresh_state)
end

function handle_event(event)
    obs.script_log(obs.LOG_INFO, "enter handle event")
    if event == obs.OBS_FRONTEND_EVENT_EXIT then
	handle_exit()
    elseif event == obs.OBS_FRONTEND_EVENT_SCENE_COLLECTION_CHANGED then
	obs.script_log(obs.LOG_INFO, "Frontend_scene_collection_changed")
        list_sources()
    elseif event == obs.OBS_FRONTEND_EVENT_SCENE_CHANGED then
	obs.script_log(obs.LOG_INFO, "Frontend_scene_changed")
	local source = obs.obs_frontend_get_current_scene()
	new_scene(source, true)
	obs.obs_source_release(source)
    elseif event == obs.OBS_FRONTEND_EVENT_PREVIEW_SCENE_CHANGED then
	obs.script_log(obs.LOG_INFO, "Frontend_scene_preview_changed")
        local source = obs.obs_frontend_get_current_preview_scene()
	new_scene(source, false)
        obs.obs_source_release(source)
    end
end

function get_scene_items(scene, source_names)
    local items = obs.obs_scene_enum_items(scene)

    for _, item in pairs(items) do
	local item_source = obs.obs_sceneitem_get_source(item)
	local item_name = obs.obs_source_get_name(item_source)
	local item_number = obs.obs_sceneitem_get_id(item)
	local source_type = obs.obs_source_get_type(item_source)
	if obs.obs_sceneitem_visible(item) then
	    if obs.obs_sceneitem_is_group(item) then
--              obs.script_log(obs.LOG_INFO, "group item: " .. item_name)
		group_scene = obs.obs_sceneitem_group_get_scene(item)
		get_scene_items(group_scene, source_names)
	    elseif source_type == 3 then
--              obs.script_log(obs.LOG_INFO, "scene item: " .. item_name)
		get_source_items(item_source, source_names)
	    else
		table.insert(source_names, item_name)
--              obs.script_log(obs.LOG_INFO, "item: " .. item_name .. "," .. item_number)
	    end
	end
    end

    obs.sceneitem_list_release(items)
end

function get_source_items(source, source_names)
	local scene = obs.obs_scene_from_source(source)
	get_scene_items(scene, source_names);
end

function new_scene(source, isprogram)
	local program_name = obs.obs_source_get_name(source)
    if program_name ~= nil then
        if isprogram then
--            obs.script_log(obs.LOG_INFO, "PROGRAM")
        else
--           obs.script_log(obs.LOG_INFO, "PREVIEW")
        end
--        obs.script_log(obs.LOG_INFO, "Scene: " .. program_name)
        
        local current_light_numbers = {}
        local current_source_names = {}
        get_source_items(source, current_source_names)
        for _, source_name in ipairs(current_source_names) do
            local light_number = obs.obs_data_get_int(working_settings, source_name)
            if light_number ~= 0 then
                current_light_numbers[light_number] = true
            end
        end
        return current_light_numbers
    else
        return nil
    end
end

function refresh_state()
    local source = obs.obs_frontend_get_current_scene()
    local program_lights = new_scene(source, true)
    obs.obs_source_release(source)
   
    local source = obs.obs_frontend_get_current_preview_scene()
    local preview_lights = new_scene(source, false)
    obs.obs_source_release(source)
    
    local new_cmdline = ""
	local tallystring = ""
    
    for i=1,NUMBER_OF_LIGHTS do
        if program_lights ~= nill then
	    if preview_lights ~= nil then
	        if program_lights[i] ~= nil then
		    tallystring = tallystring .. "1"
		elseif preview_lights[i] ~= nil then
		    tallystring = tallystring .. "2"
		else
		    tallystring = tallystring .. "0"
		end
	    else
		if program_lights[i] ~= nil then
		    tallystring = tallystring .. "1"
		else
		    tallystring = tallystring .. "0"
		end
	    end
	end
    end
    if tallystring ~= current_tallystring then
	local fullcmd = script_path() .. tallycli .. " " .. tallystring
	obs.script_log(obs.LOG_INFO, "tallystring: " .. tallystring)
        os.execute(fullcmd)
	current_tallystring = tallystring
    end
end

function handle_exit()
    local fullcmd = script_path() .. tallycli .. " 00000000"
    os.execute(fullcmd)
end
